/*
 * $Id: GestureTrainer.java,v 1.10 2000/12/02 19:04:06 hwawen Exp $
 *
 * Copyright (c) 1998 The Regents of the University of California.
 * All rights reserved.  See the file COPYRIGHT for details.
 */
package diva.apps;

import diva.canvas.CompositeFigure;
import diva.canvas.Figure;
import diva.canvas.FigureLayer;
import diva.canvas.event.LayerEvent;
import diva.canvas.toolbox.BasicFigure;

import diva.sketch.BasicInterpreter;
import diva.sketch.BasicSketchController;
import diva.sketch.JSketch;
import diva.sketch.SketchPane;
import diva.sketch.StrokeSymbol;
import diva.sketch.features.FEUtilities;
import diva.sketch.features.StrokePathLengthFE;
import diva.sketch.recognition.TimedStroke;
import diva.sketch.recognition.TrainingModel;
import diva.sketch.recognition.TrainingParser;
import diva.sketch.recognition.TrainingWriter;
import diva.sketch.toolbox.StrokeFilter;
import diva.sketch.toolbox.ApproximateStrokeFilter;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.awt.geom.Rectangle2D;

import javax.swing.AbstractAction;
import javax.swing.BorderFactory;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JList;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.ListModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.filechooser.FileFilter;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.InputStreamReader;
import java.io.IOException;
import java.io.OutputStream;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.zip.GZIPInputStream;
import java.util.zip.GZIPOutputStream;

/**
 * GestureTrainer is a gesture training program which provides an editor
 * for an user to sketch gestures and assign type names.  After
 * the user is done with entering gesture samples, these samples are
 * trained to provide recognition.  Then a sketch board is launched
 * with gesture recognition engine running.  The user can sketch some
 * gestures here to test the recognition.  If a gesture is recognized,
 * its type will be written across the gesture.
 *
 * @author Heloise Hse      (hwawen@eecs.berkeley.edu)
 * @version $Revision: 1.10 $
 */
public class GestureTrainer extends JFrame {
    /**
     * If this flag is true, generate variations of strokes from the training
     * data.
     */
    public static boolean SYNTHESIZE = true;
    
    /**
     * The default width of the editor.
     */
    public final static int WIDTH = 600;

    /**
     * The default height of the editor.
     */
    public final static int HEIGHT = 700;

    /**
     * The name given to the default pane.
     */
    private final static String UNTITLED = "Untitled";

    /**
     * ".tc" extension for training files.
     */
    public final static String TC = "tc";

    /**
     * ".tc.gz" extension for compressed training files.
     */
    public final static String TC_GZ = "tc.gz";

    /**
     * Used on the menu item and by the SaveAction to distinguish
     * which kind of save request has been made (save to .tc,
     * uncompressed).
     */
    private final static String SAVE = "Save";

    /**
     * Used on the menu item and by the SaveAction to distinguish
     * which kind of save request has been made (save to .tc.gz,
     * compressed).
     */
    private final static String SAVE_AS_GZIP = "Save As GZIP";
    
    /**
     * Each type of gesture is trained on a different pane.
     * When a user want to switch to view a different type 
     * of gesture, the corresponding pane will be displayed.
     * This hash map uses the gesture type name as the key
     * to access the pane.
     */
    private HashMap _panes = new HashMap();

    /**
     * A canvas for sketch input.
     */
    private JSketch _jsketch = null;

    /**
     * A text field for user to specify the gesture type
     * for the set of gestures to be trained.
     */
    private JTextField _inTextField = null;

    /**
     * A text area that displays messages from the
     * program.  For example, error messages or
     * messages to indicate progress.
     */
    private JTextArea _messageBox = new JTextArea();

    /**
     * The name of the pane that's currently showing.  The name is
     * actually the name of the training class, but it is also used to
     * reference the pane that the gestures of this training class are
     * drawn on.
     */
    private String _currentPaneName;

    /**
     * A window that contains a list of the names of the training
     * classes in the training set.
     */
    private TrainingSetWindow _tsWindow = null;

    /**
     * Write a training model to a file.
     */
    private TrainingWriter _writer = new TrainingWriter();
    
    /**
     * Load a training file into a training model data structure.
     */
    private TrainingParser _parser = new TrainingParser();

    /**
     * Launch a sketch pad when the "train" button has been pressed.
     * This pad allows the user to test out the training gestures with
     * recognition on.
     */
    private SketchPad _sketchPad = null;

    /**
     * Used to filter synthesized strokes.
     */
    private StrokeFilter _strokeFilter = new ApproximateStrokeFilter();
    private StrokePathLengthFE _pathlengthFE = new StrokePathLengthFE();
    
    /**
     * Exit the program when a window closing event is received.
     */
    private WindowListener windowListener = new WindowAdapter() {
        public void windowClosing(WindowEvent e) {
            System.exit(0);
        }
    };
    
    public static void main(String argv[]){
        GestureTrainer t = new GestureTrainer();
        t.setVisible(true);
    }

    /**
     * Create a gesture training editor.
     */
    public GestureTrainer(){

        super("Gesture Trainer");

        // Create the menubar and set it
        setJMenuBar(createMenuBar());

        // Close the window on any window event
        addWindowListener(windowListener);

        // add the default UNTITLED pane into the hash map.
        SketchPane pane = makePane();
        _jsketch = new JSketch(pane);
        _panes.put(UNTITLED, pane);
        
        // Initialize widget layout
        initLayout();

        // Create the list window
        _tsWindow = new TrainingSetWindow();
        _tsWindow.setLocation(WIDTH, 10);
        _tsWindow.add(UNTITLED);
        _tsWindow.select(0);
        _tsWindow.setVisible(true);
    }

    /**
     * Initialize the widget layout of the editor.
     */
    private void initLayout(){
        getContentPane().setLayout(new BorderLayout());
        getContentPane().add("Center", _jsketch);

        JPanel p1 = new JPanel(new BorderLayout());
        p1.setBorder(BorderFactory.createEmptyBorder(3,3,3,3));
        
        JPanel pcenter = new JPanel(new BorderLayout());
        pcenter.setBorder(BorderFactory.createTitledBorder("Training Class Type"));
        JPanel pcentersub = new JPanel(new BorderLayout());
        _inTextField = new JTextField(10);
        _inTextField.setToolTipText("Enter gesture type here");
        JButton addButton = new JButton("Add");
        addButton.addActionListener(new AddTrainingClassAction());
        addButton.setToolTipText("Add gestures to the training set");
        pcentersub.add("Center",_inTextField);
        pcentersub.add("East",addButton);
        pcenter.add("Center", pcentersub);

        JPanel peast = new JPanel(new FlowLayout());
        peast.setBorder(BorderFactory.createTitledBorder("Finish"));
        JButton trainButton = new JButton("Train");

        trainButton.addActionListener(new TrainAction());
        trainButton.setToolTipText("Train on the training set");
        peast.add(trainButton);
        p1.add("Center", pcenter);
        p1.add("East", peast);
        JScrollPane scroll = new JScrollPane(_messageBox);
        scroll.setBorder(BorderFactory.createTitledBorder("Message: "));

        JPanel p2 = new JPanel(new GridLayout(2,1,0,0));
        p2.setBorder(BorderFactory.createEtchedBorder());
        p2.add(p1);
        p2.add(scroll);
        getContentPane().add("South", p2);

        setSize(WIDTH, HEIGHT);
        setLocation(0, 0);
    }


    /**
     * Create the menubar.
     */
    private JMenuBar createMenuBar () {
        JMenuBar menubar;
        JMenu menuFile;
        //JMenu menuEdit;
        JMenuItem itemLoad, itemSave, itemSaveAsGZIP, itemExit;
        //JMenuItem itemUndo, itemRedo, itemRemoveAll;

        // Create the menubar and menus
        menubar = new JMenuBar();

        menuFile = new JMenu("File");
        menuFile.setMnemonic('F');
        //menuEdit = new JMenu("Edit");
        //menuEdit.setMnemonic('E');

        // Create the menu items
        itemLoad = menuFile.add(new LoadAction("Load"));
        itemLoad.setToolTipText("Load training data");
        itemSave = menuFile.add(new SaveAction(SAVE));
        itemSave.setToolTipText("Save training data");
        itemSaveAsGZIP = menuFile.add(new SaveAction(SAVE_AS_GZIP));
        itemSaveAsGZIP.setToolTipText("Save training data as a GZIP file");
        itemExit = menuFile.add(new ExitAction("Exit"));
        itemExit.setToolTipText("Exit the program");
        /*
        itemUndo = menuEdit.add(new UndoAction("Undo"));
        itemUndo.setToolTipText("Undo last gesture");
        itemRedo = menuEdit.add(new RedoAction("Redo"));
        itemRedo.setToolTipText("Redo last gesture");
        itemRemoveAll = menuEdit.add(new RemoveAllAction("Remove All"));
        itemRemoveAll.setToolTipText("Remove all gestures in the window");
        */
        // Build the menus
        menubar.add(menuFile);
        //menubar.add(menuEdit);

        return menubar;
    }

    private SketchPane makePane(){
        BasicSketchController bsc = new BasicSketchController();
        SketchInterpreter interpreter = new SketchInterpreter(bsc);
        bsc.setForegroundInterpreter(interpreter);
        SketchPane pane = new SketchPane(bsc);
        return pane;
    }
    
    /**
     * Collect the gestures on the screen to be a set of examples
     * of a training class.
     */
    private class AddTrainingClassAction extends AbstractAction {
        public void actionPerformed(ActionEvent e) {
            // add the gestures on the screen to the set and add its
            // name to the control box.
            String name = _inTextField.getText().trim();
            if(!validName(name)){
                return;
            }
            // if a new training class is added, remove UNTITLED
            // pane from the hash map, add the pane to the hash
            // map using the new name, create a new empty pane
            // and add it to the hash map using UNTITLED as key.
            if(_currentPaneName.equals(UNTITLED)){
                SketchPane pane = (SketchPane)(_panes.remove(UNTITLED));
                _panes.put(name, pane);
                SketchPane newPane = makePane();
                _panes.put(UNTITLED, newPane);
                _tsWindow.add(name);
            }
            else {
                // this case happens when the user adds gestures to an
                // existing training class and changes the name of the
                // class
                SketchPane pane =
                    (SketchPane)(_panes.remove(_currentPaneName));
                _panes.put(name,pane);
                _tsWindow.changeNameTo(_currentPaneName, name);
                _tsWindow.select(0);
            }
        }
    }

    /**
     * Exit the program.
     */
    private class ExitAction extends AbstractAction {
        public ExitAction(String s) {
            super(s);
        }
        public void actionPerformed(ActionEvent e) {
            System.exit(0);
        }
    }

    /**
     * Load a training file.
     */
    private class LoadAction extends AbstractAction {
        public LoadAction(String s) {
            super(s);
        }
        public void actionPerformed(ActionEvent e) {
            JFileChooser chooser =
                new JFileChooser(System.getProperty("user.dir"));
            TCFilter filter = new TCFilter();
            chooser.addChoosableFileFilter(filter);
            chooser.setFileFilter(filter);
            int returnVal = chooser.showOpenDialog((Component)e.getSource());
            if(returnVal == JFileChooser.APPROVE_OPTION){
                printMessage("Loading training file: "
                        + chooser.getSelectedFile().getName() + "\n");
                try{
                    File file = chooser.getSelectedFile();
                    String fileName = file.getName();
                    TrainingModel model = null;
                    if(fileName.endsWith(TC)){
                        model = (TrainingModel)_parser.parse(new java.io.FileReader(file));
                    }
                    else if(fileName.endsWith(TC_GZ)){
                        //tmp solution: decompress the gzip file and
                        //write the contents to a tmp file, then parse
                        //the tmp file.
                        // create temporary and mark "delete on exit"   
                        File tmpf = File.createTempFile(fileName,
                                null, file.getParentFile());
                        tmpf.deleteOnExit();
                        System.err.println("temp file = " + tmpf);
                        //output decompressed content to temporary file
                        GZIPInputStream gin =
                            new GZIPInputStream(new FileInputStream(file));
                        BufferedReader br =
                            new BufferedReader(new InputStreamReader(gin));
                        BufferedWriter bw =
                            new BufferedWriter(new FileWriter(tmpf.getPath()));
                        String s = null;
                        while ((s = br.readLine()) != null) {
                            bw.write(s);
                        }
                        br.close();
                        bw.close();
                        model = (TrainingModel)_parser.parse(new java.io.FileReader(tmpf.getName()));                
                    }
                    else{
                        String s = "Error: Invalid file name, " + fileName
                            + ".\n File must have .tc or .tc.gz extension.\n";
                        printMessage(s);
                        return;
                    }
                    // set up the panes
                    if(model != null){
                        modelToPanes(model);
                    }
                }
                catch(Exception ex) {
                    printMessage("Loading exception!\n");
                    ex.printStackTrace();
                }
            }
        }
    }

    /**
     * Given the training model, for each type in the model create a
     * pane.  Add the examples of a type as figures into the
     * corresponding pane.  If the type already exists in the current
     * training set, just add the examples as figures to the existing
     * pane.
     */
    private void modelToPanes(TrainingModel model){
        BasicSketchController controller =
            (BasicSketchController)_jsketch.getSketchPane().getSketchController();
        Color color = controller.getPenColor();
        float width = controller.getLineWidth();
        
        for(Iterator types = model.types(); types.hasNext();){
            String type = (String)types.next();
            SketchPane p = (SketchPane)_panes.get(type);
            if(p == null){
                p = makePane();
                _panes.put(type, p);
                _tsWindow.add(type);                
            }
            for(Iterator examples = model.positiveExamples(type);
                examples.hasNext();){
                TimedStroke stroke = (TimedStroke)examples.next();
                StrokeSymbol symbol = new StrokeSymbol(stroke, color, null, width);
                BasicFigure f = new BasicFigure(symbol.getStroke());
                f.setUserObject(symbol);
                f.setStrokePaint(symbol.getOutline());
                f.setLineWidth(symbol.getLineWidth());        
                f.setInteractor(controller.getSelectionInteractor());
                Figure cf = addDot(f);
                p.getForegroundLayer().add(cf);
            }
        }
    }

    /**
     * The index used to store the dot figure in a composite figure.
     * The dot is used to denote the starting point of a stroke.
     */
    private final static int DOT_INDEX = 0;
    /**
     * The index used to store the stroke figure in a composite
     * figure.  The composite figure also stores a dot figure which is
     * used to denote the starting point of this stroke.
     */
    private final static int FIGURE_INDEX = 0;
    
    /**
     * Return a composite figure which contains the specified figure
     * and a dot at the start of the stroke of this figure).
     */
    private Figure addDot(Figure f){
        CompositeFigure cf = new CompositeFigure();
        BasicFigure dotFigure = new BasicFigure(new Rectangle(4,4));
        dotFigure.setFillPaint(Color.orange);
        StrokeSymbol symbol = (StrokeSymbol)f.getUserObject();
        TimedStroke stroke = (TimedStroke)symbol.getStroke();
        dotFigure.translate(stroke.getX(0), stroke.getY(0));
        cf.add(DOT_INDEX, dotFigure);
        cf.add(FIGURE_INDEX, f);
        return cf;
    }
    
    /**
     * Save the training classes to a file.
     */
    private class SaveAction extends AbstractAction {
        String _name;
        
        public SaveAction(String s) {
            super(s);
            _name = s;
        }
        
        public void actionPerformed(ActionEvent e) {
            JFileChooser chooser =
                new JFileChooser(System.getProperty("user.dir"));
            TCFilter filter = new TCFilter();
            chooser.addChoosableFileFilter(filter);
            chooser.setFileFilter(filter);
            int returnVal = chooser.showSaveDialog((Component)e.getSource());
            if(returnVal == JFileChooser.APPROVE_OPTION){
                File file = chooser.getSelectedFile();
                if(_name == SAVE_AS_GZIP){
                    //make sure the file extension is .tc.gz
                    String ext = "."+TC_GZ;
                    if(!file.getName().endsWith(ext)){
                        file = new File(file.getName()+ext);
                    }
                }
                else {
                    if(!file.getName().endsWith(TC)){
                        file = new File(file.getName()+"."+TC);
                    }
                }
                printMessage("Saving to: " + file.getName() + "\n");
                try{
                    TrainingModel model = new TrainingModel();
                    genTrainingModel(model);
                    
                    // Save to file
                    OutputStream out = new FileOutputStream(file);
                    if(_name == SAVE_AS_GZIP){
                        // Compressed
                        out = new GZIPOutputStream(out);
                    }
                    // Write out data
                    _writer.writeModel(model, out);
                    // Flush output
                    out.flush();
                    // Close the stream
                    out.close();
                    printMessage("done saving\n");
                }
                catch(IOException ex) {
                    printMessage("File saving exception!\n");
                    ex.printStackTrace();
                }
            }
        }
    }

    /**
     * Generate a training model data structure from the training
     * classes.  Each training class is just a pane with figures.  So
     * we would take all of the panes and dump their figures into
     * TimedStroke objects and add these objects as examples into the
     * training model.
     */
    private void genTrainingModel(TrainingModel model){
        for(Iterator types = _panes.keySet().iterator(); types.hasNext();){
            String type = (String)types.next();
            if(type == UNTITLED){
                continue;
            }
            SketchPane pane = (SketchPane)(_panes.get(type));
            for(Iterator figures = pane.getForegroundLayer().figures();
                figures.hasNext();){
                CompositeFigure cf = (CompositeFigure)figures.next();
                Figure f = cf.get(FIGURE_INDEX);
                StrokeSymbol symbol = (StrokeSymbol)f.getUserObject();
                TimedStroke stroke = (TimedStroke)symbol.getStroke();
                model.addPositiveExample(type, stroke);
            }
        }
    }

    /**
     * Train a classifier using the training classes and open an
     * editor to test out the recognition.
     */
    private class TrainAction extends AbstractAction {
        public void actionPerformed(ActionEvent e) {
            TrainingModel model = new TrainingModel();
            if(SYNTHESIZE){
                //collect TimedStrokes for each class and generate variations.
                for(Iterator types = _panes.keySet().iterator(); types.hasNext();){
                    String type = (String)types.next();
                    if(type == UNTITLED){
                        continue;
                    }
                    ArrayList strokes = new ArrayList();
                    int maxVertices = 0;
                    SketchPane pane = (SketchPane)(_panes.get(type));
                    for(Iterator figures = pane.getForegroundLayer().figures();
                        figures.hasNext();){
                        CompositeFigure cf = (CompositeFigure)figures.next();
                        Figure f = cf.get(FIGURE_INDEX);
                        StrokeSymbol symbol = (StrokeSymbol)f.getUserObject();
                        TimedStroke stroke = (TimedStroke)symbol.getStroke();
                        model.addPositiveExample(type, stroke);//add original stroke to the training set
                        if(stroke.getVertexCount() > maxVertices){
                            maxVertices = stroke.getVertexCount();
                        }
                        strokes.add(stroke);
                    }
                    System.out.println("maxVertices = " + maxVertices);
                    ArrayList interpStrokes = generateStrokes(maxVertices, strokes);
                    System.out.println("num of interpolated strokes = " + interpStrokes.size());
                    Generator g = new Generator(interpStrokes, maxVertices);
                    for(int i=0; i< 50; i++){
                        TimedStroke tstroke = g.genStroke();
                        tstroke = _strokeFilter.apply(tstroke);
                        model.addPositiveExample(type, tstroke);//add synthesized shape to the training set
                    }
                }
            }
            else {
                genTrainingModel(model);
            }
            
            if(model.getTypeCount()==0){
                printMessage("No gestures to train.");
                return;
            }
            if(_sketchPad == null){
                _sketchPad = new SketchPad();
            }
            _sketchPad.train(model);
            _sketchPad.show();
        }
    }

    private ArrayList generateStrokes(int numVertices, ArrayList strokes){
        ArrayList list = new ArrayList();
        for(Iterator i = strokes.iterator(); i.hasNext();){
            TimedStroke s = (TimedStroke)i.next();
            if(s.getVertexCount() != numVertices){
                TimedStroke ns = interpolateStroke(numVertices, s);
                list.add(ns);
            }
            else {
                list.add(s);
            }
        }
        return list;
    }

    private TimedStroke interpolateStroke(int numPoints, TimedStroke s){
        double pathLength = _pathlengthFE.apply(s);
        double segmentLength = pathLength/(numPoints-1);
        TimedStroke newStroke = new TimedStroke(numPoints);
        double cur_x = s.getX(0);
        double cur_y = s.getY(0);
        newStroke.addVertex((float)cur_x,(float)cur_y,s.getTimestamp(0));
        double next_x, next_y, prev_dist;
        prev_dist = 0;
        int index=1;
        boolean flag = false;
        while((index < s.getVertexCount())&&
                (newStroke.getVertexCount()<numPoints)){
            next_x = s.getX(index);
            next_y = s.getY(index);
            double dist = 0;
            if(!flag){
                dist = FEUtilities.distance(cur_x, cur_y, next_x, next_y) + prev_dist;
            }
            else {
                flag = false;
                //this is when we've advanced the "next" pointer
                double start_x = s.getX(index-1);
                double start_y = s.getY(index-1);
                dist = FEUtilities.distance(start_x, start_y, next_x, next_y) + prev_dist;
            }
            if(dist==segmentLength){
                float x = (float)next_x;
                float y = (float)next_y;
                newStroke.addVertex(x, y, s.getTimestamp(index));
                cur_x = next_x;
                cur_y = next_y;
                prev_dist = 0;
                index++;
            }
            else if(dist > segmentLength){//interpolate point
                double ratio = segmentLength/dist;
                float x = (float)(cur_x + (next_x-cur_x)*ratio);
                float y = (float)(cur_y + (next_y-cur_y)*ratio);
                newStroke.addVertex(x, y, s.getTimestamp(index));
                cur_x = x;
                cur_y = y;
                prev_dist = 0;                
            }
            else{//advance "next" pointer
                index++;
                prev_dist = dist;
                flag = true;
                continue;
            }
        }
        double p = _pathlengthFE.apply(newStroke);
        if(newStroke.getVertexCount() == (numPoints-1)){
            float x = (float)s.getX(s.getVertexCount()-1);
            float y = (float)s.getY(s.getVertexCount()-1);
            long t = s.getTimestamp(s.getVertexCount()-1);
            newStroke.addVertex(x,y,t);
        }
        else if(newStroke.getVertexCount() < (numPoints-2)){
            System.out.println("INTERPOLATION ERROR!!!");
        }
        return newStroke;

    }

    /**
     * Check if the training class name entered is valid.
     * Return false if the name is empty, or named UNTITLED, or
     * there's a training class with the same name already.
     */
    private boolean validName(String s){
        if(s.length() == 0){
            printMessage("Name field is empty. Must assign a gesture name.\n");
            return false;
        }
        else if(s.compareTo(UNTITLED)==0){
            printMessage(s + " is not a valid gesture name.  Please enter another name.\n");
            return false;
        }
        else if((_panes.get(s))!= null){
            printMessage(s + " already exists.  Please use a different name.\n");
            return false;
        }
        else {
            return true;
        }
    }

    /**
     * Display the string in the message box.
     */
    private void printMessage(String s) {
        _messageBox.append(s);
    }

    /**
     * Set the current display to the pane specified by the name.
     * Each pane is associated with a name, so for example, if the
     * string is "square", retrieves the pane from the hash map and
     * display it.
     */
    private void setCurrentPane(String s){
        SketchPane p = (SketchPane)(_panes.get(s));
        if(p == null){
            printMessage("Error: There's no training class named " + s);
        }
        else {
            _currentPaneName = s;
            _inTextField.setText(_currentPaneName);
            _jsketch.setSketchPane(p);
            p.getForegroundLayer().repaint();
        }
    }

    /**
     * This window displays a list of the names of the training
     * classes that are currently in the training set.  There will
     * always be an UNTITLED item in the list.  When a user selects a
     * training class, the gestures in the class will be shown on the
     * main editor.  The user can also select a training class and
     * delete it from the training set b pressing the delete button in
     * this window.
     */
    private class TrainingSetWindow extends JFrame {
        public final static String TITLE = "Training Classes";
        public final static int WIDTH = 250;
        public final static int HEIGHT = 400;
        private JList _list;
        private JButton _delete;
        
        public TrainingSetWindow(){
            super(TITLE);
            setSize(WIDTH,HEIGHT);
            getContentPane().setLayout(new BorderLayout());
            
            ListModel model = new DefaultListModel();
            _list = new JList(model);
            _list.addListSelectionListener(new LocalListSelectionListener());
            getContentPane().setBackground(Color.lightGray);
            JScrollPane scroll = new JScrollPane(_list);
            scroll.setBorder(BorderFactory.createTitledBorder("Gesture Types"));
            getContentPane().add("Center", scroll);
            JPanel p = new JPanel(new FlowLayout(FlowLayout.CENTER));
            _delete = new JButton("Delete");
            _delete.setToolTipText("Delete the selected training class");
            _delete.addActionListener(new DeleteAction());
            p.add(_delete);
            getContentPane().add("South", p);
        }

        /**
         * If the selected page is not the untitled page, delete
         * it from the document.
         */
        private class DeleteAction extends AbstractAction {
            public void actionPerformed(ActionEvent e) {
                String s = (String)_list.getSelectedValue();
                if(s.equals(UNTITLED)){
                    printMessage("Cannot delete Untitled page, because it is the default page.\n");
                    return;
                }
                else if(s != null){
                    remove(s);
                }
            }
        }

        /**
         * When the selection in the list changes, switch the current
         * pane (on display) to the selected pane.
         */
        private class LocalListSelectionListener
            implements ListSelectionListener {
            
            public void valueChanged(ListSelectionEvent e){
                if(e.getSource() == _list){
                    if(!e.getValueIsAdjusting()){
                        String s = (String)_list.getSelectedValue();
                        int i = _list.getSelectedIndex();
                        if(i > -1){
                            setCurrentPane(s);
                        }
                        if(s.equals(UNTITLED)){
                            _delete.setEnabled(false);
                        }
                        else {
                            _delete.setEnabled(true);
                        }
                    }
                }
            }
        }

        public synchronized void add(String s){
            if(s.equals(UNTITLED)){
                ((DefaultListModel)(_list.getModel())).insertElementAt(s, 0);
            }
            else{
                ((DefaultListModel)(_list.getModel())).addElement(s);
            }
            select(0);
        }

        public void select(int i){
            _list.setSelectedIndex(i);
            String s = (String)_list.getSelectedValue();
            setCurrentPane(s);
        }
        
        /**
         * Remove the training class from the training set and remove
         * its name from the list.  Put selection on the next class.
         * If there's no next class, select the previous class.
         */
        public void remove(String s){
            _panes.remove(s);
            DefaultListModel model = (DefaultListModel)_list.getModel();
            int index = model.indexOf(s);
            model.removeElement(s);
            if(index < model.getSize()){
                select(index);
            }
            else {
                index--;
                if(index > -1){
                    select(index);
                }
            }
        }

        public void changeNameTo(String oldName, String newName){
            DefaultListModel model = (DefaultListModel)_list.getModel();
            int index = model.indexOf(oldName);
            model.setElementAt(newName, index);
        }
    }
    
    /**
     * Filters out the files which don't have ".tc" or ".tc.gz"
     * extensions.
     */
    private class TCFilter extends FileFilter {
        public TCFilter(){}
        
        // Accept all directories and (tc) files.
        public boolean accept(File f) {
            if(f.isDirectory()) {
                return true;
            }
            String s = f.getName();
            int i = s.lastIndexOf('.');
            if(i > 0 &&  i < s.length() - 1) {
                String extension = s.substring(i+1).toLowerCase();
                if (TC.equals(extension)){//look for .tc
                    return true;
                }
                else{//look for .tc.gz
                    int j = s.lastIndexOf(".", i-1);
                    if(j>0&& j<s.length()-1){
                        extension = s.substring(j+1).toLowerCase();
                        if(TC_GZ.equals(extension)){
                            return true;
                        }
                    }
                }
            }
            return false;
        }
    
        // The desctiption of this filter
        public String getDescription() {
            return "Training files (*.tc) or (*.tc.gz)";
        }
    }

    private class SketchInterpreter extends BasicInterpreter {
        public SketchInterpreter (BasicSketchController c){
            super(c);
        }
                
        public void mouseReleased (LayerEvent evt) {
            super.mouseReleased(evt);
            TimedStroke curStroke = getCurrentStroke();
            BasicFigure curFigure = getCurrentFigure();
            BasicSketchController controller =
                (BasicSketchController)getController();
            FigureLayer layer =
                controller.getSketchPane().getForegroundLayer();
            Figure cf = addDot(curFigure);
            layer.remove(curFigure);
            layer.add(cf);

        }
    }
}


