/*
 * $Id: SketchPad.java,v 1.4 2000/11/16 00:27:18 hwawen Exp $
 *
 * Copyright (c) 1998 The Regents of the University of California.
 * All rights reserved.  See the file COPYRIGHT for details.
 */
package diva.apps;

import diva.canvas.CompositeFigure;
import diva.canvas.FigureLayer;
import diva.canvas.event.LayerEvent;
import diva.canvas.toolbox.BasicFigure;
import diva.canvas.toolbox.LabelFigure;

import diva.sketch.BasicInterpreter;
import diva.sketch.BasicSketchController;
import diva.sketch.JSketch;
import diva.sketch.SketchPane;
import diva.sketch.recognition.BasicStrokeRecognizer;
import diva.sketch.recognition.StrokeRecognition;
import diva.sketch.recognition.StrokeRecognitionSet;
import diva.sketch.recognition.TimedStroke;
import diva.sketch.recognition.TrainingModel;
import diva.sketch.classification.ClassifierException;

import java.awt.BorderLayout;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.geom.Rectangle2D;
import javax.swing.AbstractAction;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;

/**
 * A SketchPad provides an editor that allows the user to try out the
 * strokes created during the training session.  It has an underlying
 * recognition engine that's been trained on the strokes from the
 * training set.  After the user draws a shape, recognition is
 * performed on the shape and a label appears above the stroke
 * specifying the type and the confidence value of this shape as
 * recognized.  For example, the type may be "square" and the
 * confidence value (ranging from 0 to 100) may be "96.5".
 *
 * @author Heloise Hse      (hwawen@eecs.berkeley.edu)
 * @version $Revision: 1.4 $
 */
public class SketchPad extends JFrame {
    /**
     * The default width of the editor.
     */
    public final static int WIDTH = 600;

    /**
     * The default height of the editor.
     */
    public final static int HEIGHT = 700;

    private BasicSketchController _controller;
    private SketchPadInterpreter _interpreter;
    private SketchPane _pane;
    private JSketch _jsketch;

    /**
     * Create a SketchPad, set the title of the frame and initialize
     * the layout of the frame.  A JSketch object is placed in the
     * center of the window for the user to sketch on.  JSketch
     * contains a sketch pane which has a controller that uses a
     * SketchInterpreter to perform interpretation on user input
     * strokes.  The interpreter is trained with the set of
     * strokes entered during the training session (and can be
     * retrained when more strokes are added).
     */
    public SketchPad(){
        super("Sketch Pad");
        // set up pane, controller, and sketch interpreter.
        _controller = new BasicSketchController();
        _interpreter = new SketchPadInterpreter(_controller);
        _controller.setForegroundInterpreter(_interpreter);
        _pane = new SketchPane(_controller);
        _jsketch = new JSketch(_pane);
        getContentPane().setLayout(new BorderLayout());
        getContentPane().add("Center", _jsketch);

        JPanel buttonPanel = new JPanel();
        JButton clear = new JButton("Clear");
        clear.addActionListener(new ClearAction());
        buttonPanel.add(clear);
        getContentPane().add("South", buttonPanel);
        
        setSize(WIDTH, HEIGHT);
        setLocation(0, 0);
    }

    /**
     * Train the recognizer on the set of strokes in the given model.
     */
    public void train(TrainingModel model){
            _interpreter.train(model);
    }

    /**
     * Remove all the figures from the canvas.  This is done by
     * creating a new sketch pane.
     */
    private class ClearAction extends AbstractAction{
        public void actionPerformed(ActionEvent evt){
            _pane = new SketchPane(_controller);
            _jsketch.setSketchPane(_pane);
            _pane.getForegroundLayer().repaint();
        }
    }

    /**
     * This interpreter uses a basic recognizer to recognize strokes.
     * The recognizer is trained on the set of strokes inputted by the
     * user during the training session.
     */
    public class SketchPadInterpreter extends BasicInterpreter {
        /**
         * recognizer that can be trained with user supplied shapes.
         */
        private BasicStrokeRecognizer _recognizer;
        

        /**
         * the font used to show the recognized type and confidence value.
         */
        private Font _labelFont = new Font("Helvetica", Font.PLAIN, 12);
        
        public SketchPadInterpreter(BasicSketchController c) {
            super(c);
            _recognizer = new BasicStrokeRecognizer();
        }

        /**
         * Train the recognizer using the given model.  The default
         * BasicRecognizer uses a non-incremental classifier, so the
         * previously trained data will be discarded before starting
         * on the new one.  This is what we want here.
         */
        public void train(TrainingModel model){
            try{
                _recognizer.train(model);
            }
            catch(ClassifierException ex){
                ex.printStackTrace();
            }
        }

        public void mouseReleased (LayerEvent evt) {
            super.mouseReleased(evt);
            TimedStroke curStroke = getCurrentStroke();
            BasicFigure curFigure = getCurrentFigure();
            BasicSketchController controller = getController();
            FigureLayer layer =
                controller.getSketchPane().getForegroundLayer();
            StrokeRecognitionSet rset =_recognizer.strokeCompleted(curStroke);
            System.err.println("R = " + rset);
            StrokeRecognition r = rset.getBestRecognition();
            
            if(r != null) {
                double confidence = r.getConfidence();

                String typeID = r.getType().getID();
                System.out.println("[" + typeID + ", "
                        + confidence + "]");
                if(confidence < 0){
                    confidence = 0;
                }
                CompositeFigure cf = new CompositeFigure();
                String id = "["+ typeID + ", " +
                    String.valueOf(confidence) + "]";
                LabelFigure lf = new LabelFigure(id);
                lf.setFont(_labelFont);
                Rectangle2D box = curFigure.getBounds();

                lf.translate(box.getX(), box.getY());
                cf.add(curFigure);
                cf.add(lf);
                layer.remove(curFigure);
                layer.add(cf);
            }
        }
    }
}

