# checkStuff.tcl: Check lots of stuff in the Diva tree
#
# $Id: checkStuff.tcl,v 1.18 2000/10/04 20:59:52 michaels Exp $
#
# Copyright (c) 1998-2000 The Regents of the University of California.
# All rights reserved. See the file COPYRIGHT for details.
#



### Assumed global variables:
# debug: print debug information if on
# silent: don't print anything if on

## Leave these directories out of the checks
set uncheckedDirectories {
    junk
    scratch
    temp
    api
    Builds
    OLD
    CVS
    SCCS
}

# Check these files for copyright notices
set checkableFiles {
    *.java
    *.html
    *.tcl
}

#######################################################################
#### checkStuff
#
# Check a bunch of stuff in the Diva tree that needs to be checked.
#
proc checkStuff {DIVA} {
    global debug silent

    if !$silent {
	puts "Checking stuff in $DIVA...\n"
    }

    ### We no longer check copyrights, as they are added to the
    ### distribution automatically.
    set OLDSTUFF {
	# Check copyrights
	if !$silent {
	    puts "Directories that do not have a COPYRIGHT file..."
	}
	set temp [checkCopyright $DIVA $DIVA]
	if { $temp == "" && !$silent } {
	    puts "    None"
	}
    }

    # Check top-level directories
    if !$silent {
	puts "\nTop-level directories that are not listed in vars.mk..."
    }
    set temp [checkToplevelDirs $DIVA]
    if { $temp == "" && !$silent } {
	puts "    OK"
    }

    # Check package directories
    if !$silent {
	puts "\nSource directories that are not listed in vars.mk..."
    }
    set temp [checkPackageDirs $DIVA $DIVA]
    if { $temp == "" && !$silent } {
	puts "    none"
    }

    # Check package directories, the other way around
    if !$silent {
	puts "\nPackages listed in vars.mk with no directory..."
    }
    set temp [checkPackageDirs2 $DIVA $DIVA]
    if { $temp == "" && !$silent } {
	puts "    none"
    }

    # Check for package.html
    if !$silent {
	puts "\nPackages that have no package.html..."
    }
    set temp [checkPackageHTML $DIVA $DIVA]
    if { $temp == "" && !$silent } {
	puts "    none"
    }

    # Check makefile concistency
    if !$silent {
	puts "\nMakefile inconsistencies..."
    }
    checkMakeStuff $DIVA $DIVA

    # Check copyright notices
    if !$silent {
	puts "\nFiles that do not contain the correct copyright..."
    }
    set temp [checkFileCopyrights $DIVA $DIVA]
    if { $temp == "" && !$silent } {
	puts "    None"
    }

    # Check RCS keywords
    if !$silent {
	puts "\nFiles that contain a bogus RCS ID string..."
    }
    set temp [checkRCSIDs $DIVA $DIVA]
    if { $temp == "" && !$silent } {
	puts "    None"
    }

    # Check javadoc tags
    if !$silent {
	puts "\nFiles that are missing javadoc tags..."
    }
    set temp [checkDoctags $DIVA $DIVA]
    if { $temp == "" && !$silent } {
	puts "    None"
    }

    if !$silent {
	puts "\nChecks complete!"
    }
}

#######################################################################
#### isCheckable
#
# See if a directory is worth checking
#
proc isCheckable {dir} {
    global uncheckedDirectories
    if { [file isdirectory $dir] \
	    && [lsearch -exact \
	    [join $uncheckedDirectories " "] \
	    [file tail $dir]] < 0 } {
	return 1
    } else {
	return 0
    }
}

#######################################################################
#### checkableFiles
#
# Get the files in a directory to check for copyrights. This is anything
# that matches the checkableFiles global, and that doesn't start
# with an underscore.
#
proc checkableFiles {dir} {
    global checkableFiles
    set patt {}
    foreach p $checkableFiles {
	lappend patt [file join $dir $p]
    }
    set files {}
    foreach f [eval glob -nocomplain $patt] {
	if ![string match {_*} [file tail $f]] {
	    lappend files $f
	}
    }
    return $files
}

#######################################################################
#### checkCopyright
#
# Check for copyright file in directories. Returns a list of
# directories that do not have the right files.
#
proc checkCopyright {DIVA dir} {
    global debug silent

    set result ""
    if ![file exists [file join $dir COPYRIGHT]] {
	if !$silent {
	    puts "    [divaPath $DIVA $dir]"
	}
	lappend result $dir
    }
    foreach dir [glob -nocomplain [file join $dir *]] {
	if [isCheckable $dir] {
            append result [checkCopyright $DIVA $dir]
	}
    }
    return $result
}


#######################################################################
#### checkDoctags
#
# Check that each source file has the correct doctags somewhere in it.
#
# Return a list of files that are missing doctags
#
proc checkDoctags {DIVA dir} {
    global debug silent

    set result {}
    set tags {author version}  ;# FIXME: Add rating later
    foreach file [glob -nocomplain [file join $dir *.java]] {
	set fd [open $file]
	set text [read $fd]
	close $fd

	set missing {}
	foreach tag $tags {
	    if ![regexp @$tag $text] {
		lappend missing @$tag
	    }
	}
	if { $missing != "" } {
	    if !$silent {
		puts "    [divaPath $DIVA $file] is missing tags: \
			[join $missing {, }]"
	    }
	    lappend result $file
	}    
    }

    foreach dir [glob -nocomplain [file join $dir *]] {
	if [isCheckable $dir] {
            append result [checkDoctags $DIVA $dir]
	}
    }
    return $result
}

#######################################################################
#### checkFileCopyrights
#
# Check for presence of copyright notice in every file that
# matches checkableFiles, and that does not start with an underscore.
# Returns a list of files that do not contain a copyright.
#
proc checkFileCopyrights {DIVA dir} {
    global debug silent

    set result {}
    foreach file [checkableFiles $dir] {
	set fd [open $file]
	set text [read $fd]
	close $fd

	set pattern "Copyright \\(c\\) 1998-2000 The Regents of the University of California\.+All rights reserved\.\[ \]+See the file COPYRIGHT for details"

	if ![regexp $pattern $text] {
	    if !$silent {
		puts "    [divaPath $DIVA $file]"
	    }
	    lappend result $file
	}
    }

    foreach dir [glob -nocomplain [file join $dir *]] {
	if [isCheckable $dir] {
            append result [checkFileCopyrights $DIVA $dir]
	}
    }
    return $result
}

#######################################################################
#### checkMakeDirs
#
# Check that sub-directories are in the makefiles. This is a
# short version of checkMakeStuff for the check-in tests.
#
proc checkMakeDirs {DIVA dir} {
    global debug silent
    set cwd [pwd]
    cd $dir

    set result {}
    # Top-level is special
    if { $DIVA == $dir } {
	foreach d [getMakevar PACKAGEDIRS] {
	    eval lappend result \
		    [checkMakeDirs $DIVA [file join $DIVA $d]]
	}
	# Clean up
	set savedSilent $silent
	set silent 1
	cd $DIVA
 	clearMakevars
	parseMakefile makefile
	set silent $savedSilent
	return $result
    }

    # Read the current makefile
    set savedSilent $silent
    set silent 1
    clearMakevars
    parseMakefile [file join $dir makefile]
    set silent $savedSilent

    # Check directories
    if [checkMakevar SUBPACKAGES] {
	set subpackages [getMakevar SUBPACKAGES]
    } else {
	set subpackages {}
    }
    set exempts [getMakevar EXEMPT_PACKAGES]
    set exempts [concat $exempts [getMakevar UNRELEASED_PACKAGES]]
    
    lappend subpackages test  ;# Test directory is OK...
    foreach d [glob -nocomplain *] {
	if { [file isdirectory $d] } {
	    set pkg [join [file split [divaPath $DIVA $dir]] .].$d
	    if { [isCheckable $d]  \
		    && [file exists [file join $dir $d makefile]] \
		    && [lsearch -exact $subpackages $d] < 0 \
		    && [lsearch -exact $exempts $pkg] < 0 } {
		if !$silent {
		    puts "    [file join $dir $d]/ is not in makefile"
		}
		lappend result [file join $dir $d]
	    }
	}
    }
    # Go into subdirectories, including ones that looked bogus,
    # if they have a makefile
    foreach d [glob -nocomplain *] {
	if { [file isdirectory $d] && [isCheckable $d] \
		&& [file exists [file join $dir $d makefile]] } {
	    eval lappend result [checkMakeDirs $DIVA [file join $dir $d]]
	    cd $dir
	}
    }
    cd $dir
    return $result
}

#######################################################################
#### checkMakeStuff
#
# Check stuff in the makefiles:
#
# The SUBPACKAGES variable in the makefile match the directories
# The JSRCS in the directory match the makefile
# There are no bogus classfiles.
#
proc checkMakeStuff {DIVA dir} {
    global debug silent
    set cwd [pwd]
    cd $dir

    # Top-level is special
    if { $DIVA == $dir } {
	foreach d [getMakevar PACKAGEDIRS] {
	    checkMakeStuff $DIVA [file join $DIVA $d]
	}
	# Clean up
	set savedSilent $silent
	set silent 1
	cd $DIVA
 	clearMakevars
	parseMakefile makefile
	set silent $savedSilent
	return
    }

    # Read the current makefile
    set savedSilent $silent
    set silent 1
    clearMakevars
    parseMakefile [file join $dir makefile]
    set silent $savedSilent

    # Check directories
    if [checkMakevar SUBPACKAGES] {
	set subpackages [getMakevar SUBPACKAGES]
    } else {
	set subpackages {}
    }
    lappend subpackages test  ;# Test directory is OK...
    foreach d [glob -nocomplain *] {
	if { [file isdirectory $d] && [isCheckable $d] }  {
	    if { [lsearch -exact $subpackages $d] < 0 } {
		puts "    [file join $dir $d]/ is not in makefile"
	    }
	}
    }

    # Check source files
    if [checkMakevar JSRCS] {
	set jsrcs [getMakevar JSRCS]
    } else {
	set jsrcs {}
    }
    foreach f [glob -nocomplain *.java] {
	if { [lsearch -exact $jsrcs $f] < 0 } {
	    puts "    [file join $dir $f] is not in makefile"
	}
    }
    # Check class files
    foreach f [glob -nocomplain *.class] {
	set stem [lindex [split [file rootname $f] \$] 0]
	if { [lsearch -exact $jsrcs $stem.java] < 0 } {
	    puts "    [file join $dir $f] does not match any source files"
	}
    }

     # Check other files
    if [checkMakevar MISC_FILES] {
	set miscfiles [getMakevar MISC_FILES]
    } else {
	set miscfiles {}
    }
    foreach f [glob -nocomplain *] {
	if { ![file isdirectory $f] \
		&& ![string match *.java $f] \
		&& ![string match *.class $f] \
		&& ![string match *.dtd $f] \
		&& ![string match *.g2d $f] \
		&& ![string match *.gif $f] \
		&& ![string match *.gml $f] \
		&& ![string match *.gsa $f] \
		&& ![string match *.sml $f] \
		&& ![string match *.tc $f] \
		&& ![string match *.xml $f] \
		&& $f != "makefile" \
		&& $f != "package.html" \
		&& [lsearch -exact $miscfiles $f] < 0 } {
	    puts "    [file join $dir $f] looks bogus"
	}
    }

    # Go into subdirectories, including ones that looked bogus
    # If they don't have a makefile, complain.
    foreach d [glob -nocomplain *] {
	if { [file isdirectory $d] && [isCheckable $d] }  {
	    if ![file exists [file join $dir $d makefile]] {
		puts "    [file join $dir $d]/ does not have a makefile"
	    } else {
		checkMakeStuff $DIVA [file join $dir $d]
		cd $dir
	    }
	}
    }
    cd $dir
}

#######################################################################
#### checkPackageDirs
#
# Check that all package directories are listed in vars.mk.
#
proc checkPackageDirs {DIVA dir} {
    global debug silent
    
    set result {}

    # Get package directories
    set packages [concat \
	    [getMakevar PACKAGES] \
	    [getMakevar EXEMPT_PACKAGES] \
	    [getMakevar AUXILIARY_PACKAGES] \
	    [getMakevar UNRELEASED_PACKAGES] \
	    [getMakevar BROKEN_PACKAGES]]

    set packageDirs {}
    foreach p $packages {
	lappend packageDirs [eval file join [split $p "."]]
    }

    # Fake special dirs
    set specialDirs {}
    foreach d [getMakevar AUXILIARYDIRS] {
	lappend specialDirs [file join diva $d]
    }
    
    # Check them
    foreach d [glob -nocomplain [file join $dir *]] {
	if { [file isdirectory $d] && [isCheckable $d] } {
	    set path [divaPath $DIVA $d]
	    if { [lsearch -exact $packageDirs $path] < 0 \
		    && [lsearch -exact $specialDirs $path] < 0 } {
		if !$silent {
		    puts "    [divaPath $DIVA $d]"
		}
		lappend result [divaPath $DIVA $d]
	    }
	}
    }

    # Recurse
    foreach d [glob -nocomplain [file join $dir *]] {
	if { [file isdirectory $d] && \
		[lsearch -exact $specialDirs [divaPath $DIVA \
		[file join $dir $d]]] < 0 \
		&& [isCheckable $d] } {
	    eval lappend result \
		    [checkPackageDirs $DIVA [file join $DIVA $d]]
	}
    }
    return $result
}

#######################################################################
#### checkPackageDirs2
#
# Check that all package directories listed in vars.mk actually exist
#
proc checkPackageDirs2 {DIVA dir} {
    global debug silent
    
    set result {}

    # Get package directories
    set packages [concat \
	    [getMakevar PACKAGES] \
	    [getMakevar AUXILIARY_PACKAGES] \
	    [getMakevar UNRELEASED_PACKAGES] \
	    [getMakevar BROKEN_PACKAGES] \
	    [getMakevar EXEMPT_PACKAGES]]

    foreach p $packages {
	set path [file join $DIVA [eval file join \
		[lreplace [split $p "."] 0 0]]]
	if ![file exists $path] {
	    if !$silent {
		puts "    $p"
	    }
	    lappend result $p
	}
    }
    return $result
}

#######################################################################
#### checkPackageHTML
#
# Check that all package directories have a package.html file
#
proc checkPackageHTML {DIVA dir} {
    global debug silent
    
    set result {}

    # Get package directories
    set packages [concat \
	    [getMakevar PACKAGES] \
	    [getMakevar UNRELEASED_PACKAGES] \
	    [getMakevar BROKEN_PACKAGES] \
	    [getMakevar EXEMPT_PACKAGES]]

    foreach p $packages {
	set path [file join $DIVA [eval file join \
		[lreplace [split $p "."] 0 0]]]
	set file [file join $path package.html]
	if ![file exists $file] {
	    if !$silent {
		puts "    $p"
	    }
	    lappend result $p
	}
    }
    return $result
}


#######################################################################
#### checkRCSIDs
#
# Check that each source file has an RCS ID, and that the ID actually
# matches the file name. Currently, this checks only Java files.
#
# Return a list of files that do not contain valid IDs.
#
proc checkRCSIDs {DIVA dir} {
    global debug silent

    set result {}
    foreach file [glob -nocomplain \
	    [file join $dir *.tcl] \
	    [file join $dir *.java]] {
	set fd [open $file]
	set text [read $fd]
	close $fd

	set pattern "\\\$Id: (\[^,\]*),v"

	if ![regexp $pattern $text _ filename] {
	    if !$silent {
		puts "    [divaPath $DIVA $file] has no RCS ID"
	    }
	    lappend result $file

	} elseif { $filename != [file tail $file] } {
	    if !$silent {
		puts "    [divaPath $DIVA $file] has a bogus RCS ID"
	    }
	    lappend result $file
	}

	# Check for the bogus $Version$ thing
	if [regexp {\$Version\$} $text] {
	    if !$silent {
		puts "    [divaPath $DIVA $file] has bad ID \"\$Version\$\""
	    }
	    lappend result $file
	}	    
    }

    foreach dir [glob -nocomplain [file join $dir *]] {
	if [isCheckable $dir] {
            append result [checkRCSIDs $DIVA $dir]
	}
    }
    return $result
}

#######################################################################
#### checkToplevelDirs dir
#
# Check that top-level directories are listed in vars.mk
#
proc checkToplevelDirs {DIVA} {
    global debug silent

    set result {}
    set goodDirs [concat [getMakevar PACKAGEDIRS] \
	    [getMakevar AUXILIARYDIRS] [getMakevar OBSOLETEPACKAGEDIRS]]
    foreach dir [glob -nocomplain [file join $DIVA *]] {
	if [isCheckable $dir] {
	    if {[lsearch -exact $goodDirs [file tail $dir]] < 0 } {
		if !$silent {
		    puts "    [divaPath $DIVA $dir]"
		}
		lappend result $dir
	    }
	}
    }
    return $result
}








