# dependencies.tcl: Utilities for building Java dependencies
#
# $Id: dependencies.tcl,v 1.9 2000/05/02 00:01:28 johnr Exp $
#
# Copyright (c) 1998-2000 The Regents of the University of California.
# All rights reserved. See the file COPYRIGHT for details.
#


#######################################################################
#### generateDependencies
#
# Generate the dependencies for each file in the JSRCS
# variable of the makefile in the current directory, and
# modify the makefile accordingly.
#
proc generateDependencies {DIVA args} {
    global debug silent

    # Parse the makefile variables
    clearMakevars

    ### Not needed, as the makefile is already parsed in divadmin
    parseMakefile makefile

    # Extract -fast option
    set i [lsearch -exact $args -fast]
    if { $i >= 0 } {
	set fast 1
	set args [lreplace $args $i $i]
    } else {
	set fast 0
    }

    # Extract -recursive option
    set i [lsearch -exact $args -recursive]
    if { $i >= 0 } {
	set recursive 1
	set args [lreplace $args $i $i]
    } else {
	set recursive 0
    }

    # Get files to generate dependencies of
    # Note: at the moment any subset will lose remaining files
    if { $args != "" } {
	set jsrcs $args
    } else {
	set jsrcs [getMakevar JSRCS]
    }

    # Strip dependencies out of the makefile
    set fd [open makefile]
    set text [read $fd]
    set originalText $text
    close $fd
    regsub "\n### DEPENDENCIES.*\$" $text {} text
    append text "\n### DEPENDENCIES: DO NOT EDIT BELOW THIS LINE\n"

    # Process each Java source file
    set dir [pwd]
    puts $dir
    set root [file join [getMakevar DIVA] ..]
    if { $jsrcs != "" } {
	foreach j $jsrcs {
	    puts "    $j"
	    update
	    if [catch {
		set depends [compileDependencies $DIVA [file join $dir $j]]
	    } result] {
		global errorInfo
		puts "Failed to compile dependencies, trying fast method"
		puts $result:\b$errorInfo
		set depends [extractDependencies $DIVA [file join $dir $j]]
	    }

	    set rule "[file rootname $j].class:"
	    # Sort by path length
	    catch {unset temp}
	    foreach d $depends {
		set l [llength [file split $d]]
		lappend temp($l) $d
	    }
	    foreach l [lsort -int [array names temp]] {
		foreach d [lsort $temp($l)] {
		    append rule " \\\n\t$d"
		}
	    }
	    ### puts $rule
	    append text $rule\n\n
	}

	if { [string trim $text \n] != [string trim $originalText \n] } {
	    set fd [open makefile "w"]
	    puts $fd $text
	    close $fd
	} else {
	    puts "    (No change)"
	}
    }
    # Do subdirectories
    if [checkMakevar SUBDIRS] {
	set subdirs [getMakevar SUBDIRS]
    } elseif [checkMakevar SUBPACKAGES] {
	set subdirs [getMakevar SUBPACKAGES]
    } else {
	set subdirs {}
    }
    foreach d $subdirs {
	cd $d
	generateDependencies $DIVA
	cd ..
    }
}

#######################################################################
#### compileDependencies
#
# Given a Java source file path, return a list of files, relative
# to the source file, of files in the Diva tree that the given source
# file depends upon. Do this by compiling the file and extracting
# dependency information from the compiler's output.
#
proc compileDependencies {DIVA file} {
    global debug silent

    set dir [file dirname $file]
    cd $dir

    # Compile the file
    set javac [getMakevar JAVAC]
    set classpath [getMakevar CLASSPATH]
    set command "$javac -classpath $classpath -verbose [file tail $file]"

    if $debug {
	puts $command
    }
    if { [catch "exec $command >& tmpfile" messages] } {
	global errorInfo
	puts $errorInfo
	error $messages $errorInfo
    }

    # Parse the compiler output
    set depends [list [file tail $file]]
    set fd [open tmpfile]
    while { ![eof $fd] } {
	set line [gets $fd]
	if [regexp {loaded (.*diva[^ $]*\.class)} $line _ class] {
	    cd [file join $dir [file dirname $class]]
	    set f [file join [pwd] [file rootname [file tail $class]]].java
	    cd $dir
	    lappend depends [relativePath [file dirname $file] $f]
	}
    }
    close $fd
    return $depends
}

#######################################################################
#### extractDependencies
#
# Given a Java source file path, return a list of files, relative
# to the source file, of files in the Diva tree that the given source
# file depends upon. Do this by looking at import statements and
# the existence of files.
#
proc extractDependencies {DIVA file} {
    global debug silent

    # Extract and expand imports
    set imports [join [file split [divaPath $DIVA [file dirname $file]]] .].*
    set imports [concat $imports [parseImports $file {^diva}]]
    set imports [expandImports [file dirname $DIVA] $imports]

    # See which of them occur in the text
    set fd [open $file]
    set text [read $fd]
    close $fd
    set depends {}
    foreach f $imports {
	set classname [file rootname [file tail $f]]
	if [regexp "\[^a-zA-Z0-9\]$classname\[^a-zA-Z0-9\]" $text] {
	    lappend depends [relativePath [file dirname $file] $f]
	}
    }
    return $depends
}

#######################################################################
#### parseImports
#
# Given a Java source file path, return the arguments to the
# import statements that match the given regexp.
#
proc parseImports {file {regexp {}}} {
    set result {}
    set fd [open $file]
    while { ![eof $fd] } {
	set line [gets $fd]
	if [regexp "^import \(\[^;\]*\)" $line _ import] {
	    if { $regexp == "" || [regexp $regexp $import] } {
		lappend result $import
	    }
	}
    }
    close $fd
    return $result
}

#######################################################################
#### expandImports
#
# Given a path root (such as derived from a class-path)
# and a list of arguments to Java's import statement, return
# the absolute path names for which a source file
# can be found.
#
proc expandImports {root imports} {
    set result {}
    foreach i $imports {
	if [regexp {\*$} $i] {
	    # Process wild-card
	    set dir [eval file join [list $root] [lreplace [split $i .] end end]]
	    foreach f [glob -nocomplain [file join $dir *.java]] {
		lappend result $f
	    }
	} else {
	    # Process single import
	    set file [eval file join [list $root] [split $i .].java]
	    if [file exists $file] {
		lappend result $file
	    }
	}
    }
    return $result
}







