# documentOutline.tcl: Generate outline of HTML documents
#
# $Id: documentOutline.tcl,v 1.4 2000/05/02 00:01:28 johnr Exp $
#
# Author: John Reekie
#
# Copyright (c) 1998-2000 The Regents of the University of California.
# All rights reserved. See the file COPYRIGHT for details.
#

# Make sure we're initialized
document::init

##########################################################################
#### documentOutline
#
# Procedure to generate/modify a document skeleton from a
# TIM document model. The first argument is the name of the file
# containing the document document model; if not supplied,
# the file "_document.tim" in the current directory
# is used by default. Any additional arguments are given
# to the document model as options.
#
proc documentOutline {DIVA {file {}} args} {
    if [string match -* $file] {
        set args [lindex $args $file]
        set file {}
    }
    if { $file == ""} {
	set file [file join [pwd] _document.tim]
    } else {
	# set file [::document::expandPath $file]
	# FIXME
	set file $file
    }
    set model document[incr document::counter]
    eval document::create $model -modelpath $file $args
    set model document::$model
    set fd [open $file]
    $model parse [read $fd]
    close $fd
    $model generate
    $model destroy
}

########################################################################
#### generate
#
# Generate the HTML files. The files will be generated in the
# same directory as this model resides in. The document root will
# be named "index.html".
#
proc document::generate {ns} {
    # Make sure options are set, using pwd if necessary
    if { [$ns cget -modelpath] == "" } {
        $ns configure -modelpath [file join [pwd] _document.tim]
    }
    if { [$ns cget -directory] == "" } {
        $ns configure -directory [file dirname [$ns cget -modelpath]]
    }
    puts [$ns cget -directory]:

    # Call recursive helper method
    _generate $ns "index" [$ns cget -inline]
}

########################################################################
#### _contents
#
# Generate the out-of-line contents of a section. The argument
# is the current section. The table of contents enclosed within
# <code>&lt;!--contents--&gt;</code> and
# <code>&lt;!--/contents--&gt;</code> are replaced. If this cannot be
# found, the contents is inserted after the top-level heading.
#
proc document::_contents {ns section} {
    if { $section == "index" } {
        set context ""
    } else {
        set context $section
    }

    # Remember the current data
    set data [set ${ns}::_data]

    # For each section, recursively call _generate{} and build contents
    # at the same time
    set contents ""
    if { [$ns match section $context] != "" } {
	# append contents "<!--contents-->\n"
	append contents "<ul>\n"
	foreach sec [$ns match section $context] {
	    # Make new file
	    set subsection [concat $context $sec]
	    _generate $ns $subsection [$ns attrget $subsection -inline]

	    # Build table of contents
	    set sectitle [$ns attrget $subsection -title]
	    append contents "<li><a href=\"$sec[$ns cget -suffix]\">$sectitle</a>\n"
	}
	append contents "</ul>"
	#append contents "<!--/contents-->"
    }

    # If this is the top level, discard the contents and generate the
    # complete nested contents
    if { $context == "" } {
	set contents ""
	#append contents "<!--contents-->\n"
	append contents [string trimleft [_fullcontents $ns $section] \n]
	#append contents "<!--/contents-->"
    }

    # Restore the data
    set ${ns}::_data $data

    # Replace or insert the table of contents
    if ![substComment ${ns}::_data "contents" $contents] {
        # Failed to find it: insert after /h1
	set contents "<!--contents-->\n$contents\n<!--/contents-->"
	insertAfter ${ns}::_data "/h1" \n$contents
    }
}

########################################################################
#### _fullcontents
#
# Generate the full out-of-line contents of a section. This recursive
# procedure generates a nested list reflecting the structure
# at all sublevels of the given level. It includes the list
# delimiters but not the contents comment delimiter.
#
proc document::_fullcontents {ns section} {
    if { $section == "index" } {
        set context ""
    } else {
        set context $section
    }

    # For each section, recursively call if it also has contents
    set contents ""
    if { [$ns match section $context] != "" } {
	append contents "<ul>\n"
	foreach sec [$ns match section $context] {
	    # Build this level
	    set subsection [concat $context $sec]
	    set sectitle [$ns attrget $subsection -title]
	    append contents "<li><a href=\"$sec[$ns cget -suffix]\">$sectitle</a>\n"

	    # Build sublevel
	    #if {[$ns match section $subsection] != ""} {
		append contents [_fullcontents $ns $subsection]\n
	    #}
	}
	append contents "</ul>\n"
    }
    # Return the generated list
    return $contents
}

########################################################################
#### _extract
#
# Extract the data belonging to a section. If the section is
# not in the current document, have a look for a related file.
#
proc document::_extract {ns section level} {
    # How to know the end of this section
    set close "h$level"
    set i $level
    incr level -1
    while { $i > 0 } {
        set close "h$i|$close"
        incr i -1
    }
    # Find the section in this document
    set anchor [join $section "_"]
    if [regexp -indices "<a name=\"$anchor\">\[ \n\]*<h$level>\[^<]*$close" [set ${ns}::_data] _ 
}

########################################################################
#### _footer
#
# Generate a file footer. If the <b>-footer</b>
# option is null, do nothing. Otherwise, replace the text within
# <code>&lt;!--customfooter--&gt;</code> and
# <code>&lt;!--/customfooter--&gt;</code>.
# If it doesn't exist, insert the footer before the
# <code>&lt;/body&gt;</code> tag.
#
proc document::_footer {ns} {
    set footer [string trim [$ns cget -footer] \n]
    if ![substComment ${ns}::_data "customfooter" $footer] {
        # Failed to find it: insert before /body
	set string "<!--customfooter-->\n$footer\n<!--/customfooter-->"
	insertBefore ${ns}::_data "/body" $string\n
    }
}

########################################################################
#### _generate
#
# Helper method for <b>generate</b>. This method generates a single file.
#
proc document::_generate {ns section inline} {
    set directory [$ns cget -directory]
    set suffix [$ns cget -suffix]

    # Filename
    set filename [file join $directory \
            [lindex $section end]$suffix]
    puts "  [lindex $section end]$suffix"

    # Make a backup of the file if it already exists
    if [file exists $filename] {
        if ![file exists [file join $directory backup]] {
            file mkdir [file join $directory backup]
        }
        file copy -force $filename [file join $directory \
                backup [lindex $section end]$suffix]
    }

    # If the file exists, get its data. Otherwise create default data
    if [file exists $filename] {
        set fd [open $filename]
        set ${ns}::_data [read $fd]
        close $fd
	set originalText [set ${ns}::_data]
    } else {
        # Use the document template
        set ${ns}::_data [$ns cget -template]
	set originalText ""
    }

    # Modify the titles, the contents, the footer, and the navigation links
    _title $ns $section
    _footer $ns   ;# Note: do footer before doing contents
    _header $ns
    if $inline {
        _inline $ns $section
    } else {
        _contents $ns $section
    }
    if { $section != "index" } {
        _navigation $ns $section
    }

    # Write the data to the file if it changed
    set newText [string trim [set ${ns}::_data] "\n"]
    if { [string trim $originalText "\n"] != $newText } {
	set fd [open $filename "w"]
	puts $fd $newText
	close $fd
    }
    set ${ns}::_data ""
}

########################################################################
#### _header
#
# Generate a file header. If the <b>-header</b>
# option is null, do nothing. Otherwise, replace the text within
# <code>&lt;!--header--&gt;</code> and <code>&lt;!--/header--&gt;</code>.
# If it doesn't exist, insert the header after the
# <code>&lt;/title&gt;</code> tag.
#
proc document::_header {ns} {
    set header [string trim [$ns cget -header] \n]
    if ![substComment ${ns}::_data "header" $header] {
        # Failed to find it: insert after /title
	set string "<!--header-->\n$header\n<!--/header-->"
	insertAfter ${ns}::_data "/title" \n$string
    }
}

########################################################################
#### _inline
#
# Generate the in-line contents of a section. The argument
# is the current section. The table of contents enclosed within
# <code>&lt;!--contents--&gt;</code> and
# <code>&lt;!--/contents--&gt;</code> are replaced. If this cannot be
# found, the contents is inserted after the top-level heading.
#
# <p><b>This is BROKEN</b>
#
proc document::_inline {section} {
    if { $section == "index" } {
        set context ""
    } else {
        set context $section
    }
    set newdata ""
    set contents ""
    if { [match section $context] != "" } {
	append contents "<!--contents-->\n"
	append contents "<ul>\n"
	foreach sec [match section $context] {
	    set sectitle [attrget [concat $context $sec] -title]

	    # Generate contents
	    append contents "<li><a href=\"#$sec\">$sectitle</a>\n"

	    # Modify the section title if it exists, otherwise add new
	    # data
	    if ![regsub \
		    "(<a name=\"$sec\">\[ \n\]*)<h2>.*</h2>(\[ \n\]*</a>)" \
		    [set ${ns}::_data] \
		    "\\1<h2>$sectitle</h2>\\2" ${ns}::_data] {
		# It's not there. Insert data before footer
		regsub <!--customfooter--> [set ${ns}::_data] \
			"<a name=\"$sec\">\n \
			<h2>$sectitle</h2>\n \
			</a>\n \
			<p>\n\n\n \
			<!--customfooter-->" ${ns}::_data
	    }
	}
	append contents "</ul>\n"
	append contents "<!--/contents-->"
    }

    # Replace or insert the table of contents
    if ![regsub "<!--contents-->.*<!--/contents-->" \
            [set ${ns}::_data] $contents ${ns}::_data] {
        # Failed to find it: insert after /h1
        regsub "</h1>" [set ${ns}::_data] "</h1>\n\n\n$contents" ${ns}::_data
    }
}

########################################################################
#### _navigation
#
# Modify the navigation links. The section is the section
# of this particular file. The text between
# <code>&lt;!--navigation--&gt;</code> and
# <code>&lt;!--/navigation--&gt;</code> are replaced. If this cannot be
# found, insert the navigation links prior to the footer.
#
proc document::_navigation {ns section} {
    # Find the position of this section
    set within [lreplace $section end end]
    set sect [lindex $section end]
    set sections [$ns match section $within]
    set suffix [$ns cget -suffix]
    set images [$ns cget -images]
    set title [$ns cget -title]

    #if { [file pathtype $imagedir] == "relative" } {
    #    set idir [file join $directory $imagedir]
    #} else {
    #    set idir $imagedir
    #}
    set idir [$ns cget -imagedir]
    set nav "<br>\n"

    # Top. Always goes to top.
    append nav "<a href=\"index$suffix\"> \
            <img \
            border=0 \
            src=\"[file join $idir [lindex $images 0]]\" \
            alt=\"Top: $title\"></a>\n"

    # Previous
    set index [lsearch -exact $sections $sect]
    if { $index > 0 } {
        set prevname [lindex $sections [expr $index-1]]
        set prevfile $prevname$suffix
        set prevtitle [$ns attrget [concat $within $prevname] -title]
        append nav "<a href=\"$prevfile\"> \
                <img \
                border=0 \
                src=\"[file join $idir [lindex $images 1]]\" \
                alt=\"Previous: $prevtitle\"></a>\n"
    }
    # Up
    if { $within != "" } {
        set upfile $within$suffix
        set uptitle [$ns attrget $within -title]
        append nav "<a href=\"$upfile\"> \
                <img \
                border=0 \
                src=\"[file join $idir [lindex $images 2]]\" \
                alt=\"Up: $uptitle\"></a>\n"
    }
    # Next
    if { $index < [expr [llength $sections] - 1]} {
        set nextname [lindex $sections [expr $index+1]]
        set nextfile $nextname$suffix
        set nexttitle [$ns attrget [concat $within $nextname] -title]
        append nav "<a href=\"$nextfile\"> \
                <img \
                border=0 \
                src=\"[file join $idir [lindex $images 3]]\" \
                alt=\"Next: $nexttitle\"></a>\n"
    }
    append nav "\n<br>\n"

    # Replace or insert the navigation links
    if ![substComment ${ns}::_data "navigation" $nav] {
        # Failed to find it: insert before customfooter
	insertBefore ${ns}::_data "customfooter" $nav
    }
}

########################################################################
#### _section
#
# Generate the in-line contents of a section. The arguments
# are the current section and the heading level. The body
# and table of contents so far are contained in the protected
# variables _body and _contents.
#
proc document::_section {ns section level} {
    if { $section == "index" } {
        set context ""
    } else {
        set context $section
    }
    append _contents "<ul>\n"
    incr level
    foreach s [$ns match section $context] {
        set sect [concat $context $s]
        set sectitle [$ns attrget $sect -title]
        set anchor [join $sect "_"]

        # Generate contents
        append _contents "<li><a href=\"#$anchor\">$sectitle</a>\n"

        # Get the contents of this section and add to the data
        set data [_extract $ns $sect $level]
        append ${ns}::_data \
                "<a name=\"$sec\">\n \
                <h$level>$sectitle</h$level>\n \
                $data"
    }
    append _contents "</ul>\n"
}

########################################################################
#### _title
#
# Modify the title and heading. If they don't exist, add
# them.
#
proc document::_title {ns section} {
    if { $section != "index" } {
        set sectitle [$ns attrget $section -title]
    } else {
        set sectitle [$ns cget -title]
    }
    # Title
    if ![substTag ${ns}::_data "title" $sectitle] {
        # Failed to find title. Insert before /head
	if ![insertBefore ${ns}::_data "/head" <title>$sectitle</title>\n] {
	    puts "Error: no <title> found"
	}
    }
    # First heading
    if ![substTag ${ns}::_data "h1" $sectitle] {
        # Failed to find h1. Insert after body
	if ![insertAfter ${ns}::_data "body" \n<h1>$sectitle</h1>] {
	    puts "Error: no <body> found"
	}
    }
    # Background color
    set bgcolor [$ns cget -bgcolor]
    if ![substAttribute ${ns}::_data "body" "bgcolor" $bgcolor] {
	# Not found: add to tag
	if ![insertAttribute ${ns}::_data "body" "bgcolor" $bgcolor] {
	    puts "Error: no <body> found"
	}
    }
    # Who created this -- I did!
    set date [clock format [clock seconds]]
    if ![regsub {<meta name=\"generator\"[^>]*>} [set ${ns}::_data] \
            "<meta name=\"generator\" content=\"divadmin documentOutline\">" \
            ${ns}::_data] {
        regsub <title> [set ${ns}::_data] \
                "<meta name=\"generator\" \
                content=\"divadmin documentOutline\">\n \
                <title>" \
                ${ns}::_data
    }
}







