# divaTree.tcl: Utilities related to the Diva tree
#
# $Id: filterTree.tcl,v 1.6 2000/05/02 00:01:28 johnr Exp $
#
# Copyright (c) 1998-2000 The Regents of the University of California.
# All rights reserved. See the file COPYRIGHT for details.
#




### Assumed global variables:
# debug: print debug information if on
# silent: don't print anything if on

### Variables
#

# Any directory with one of these names is filtered out
set excludedDirs {
    junk
    scratch
    temp
    slides
}

# Any file matching one of these glob patterns is filtered out
set excludedFiles {
    *.xar
    *.xpr
    *.doc
    *.fm
    *.vsd
    *.ppt
}

#######################################################################
#### filterJunk
#
# Filter a tree in preparation for building a release. This
# proc removes the excluded directories and files from the current
# directory and proceeds to sub-directories.
# It will not run if the directory is under CVS control, on
# the grounds that this script should be run only on an
# exported tree.
#
proc filterJunk {DIVA} {
    global excludedDirs excludedFiles
    global debug silent

    if [file exists CVS] {
	error "Directory \"[pwd]\" is under CVS control -- cannot run"
    }
    set files [eval glob -nocomplain [join $excludedFiles] " "]
    if {$files != ""} {
	if !$silent {
	    puts "    Deleting files in [divaPath $DIVA [pwd]]"
	}
	foreach f $files {
	    file delete -force $f
	}
    }
    foreach f [eval glob -nocomplain [join $excludedDirs " "]] {
	if !$silent {
	    puts "    Deleting [divaPath $DIVA [file join [pwd] $f]]"
	}
	file delete -force $f
    }
    foreach f [glob -nocomplain *] {
	if [file isdirectory $f] {
	    cd $f
	    filterJunk $DIVA
	    cd ..
	}
    }
}

#######################################################################
#### filterPackages
#
# Remove any packages that are not in the argument list.
#
proc filterPackages {DIVA args} {
    global debug silent
    cd $DIVA
    if [file exists CVS] {
	error "Directory \"[pwd]\" is under CVS control -- cannot run"
    }

    if !$silent {
	puts "Keeping packages: $args"
    }
    set specialDirs [getMakevar AUXILIARYDIRS]
    set packageDirs [eval pkgDirs $DIVA $args]

    foreach dir [glob *] {
	if ![file isdirectory $dir] {
	    continue
	}
	if { [lsearch -exact $specialDirs $dir] < 0 } {
	    set dirpath [file join $DIVA $dir]
	    if { [lsearch -exact $packageDirs $dirpath] < 0 } {
		puts "    Deleting \"[divaPath $DIVA $dirpath]\""
		file delete -force $dirpath
	    } else {
		cd $dir
		eval filterDirectories $DIVA $packageDirs
		cd ..
	    }
	}
    }
}

#######################################################################
#### filterPi
#
# Remove references to Pi from any files in the argument list.
#
proc filterPi {DIVA args} {
    global debug silent

    foreach file $args {
	if ![file exists $file] {
	    puts "ERROR: File \"$file\" doesn't exist"
	}
	if !$silent {
	    puts "    De-pi $file"
	}
	set fd [open $file]
	set text [read $fd]
	close $fd

	set pattern "<a href=\"\[\./]*admin/index\.html\[^>\]*><\[^>\]*></a>"
	set result [regsub $pattern $text "" text]

	if !$result {
	    puts "!!! Failed to de-pi $file!!!"
	}

	set fd [open $file "w"]
	puts $fd $text
	close $fd
    }
}

#######################################################################
#### filterDirectories
#
# Remove any directories that are not in the argument list.
#
proc filterDirectories {DIVA args} {
    global debug silent

    if [file exists CVS] {
	error "Directory \"[pwd]\" is under CVS control -- cannot run"
    }
    foreach dir [glob *] {
	if ![file isdirectory $dir] {
	    continue
	}
	set dirpath [file join [pwd] $dir]
	if { [lsearch -exact $args $dirpath] < 0 } {
	    puts "    Deleting \"[divaPath $DIVA $dirpath]\""
	    file delete -force $dirpath
	} else {
	    cd $dir
	    eval filterDirectories $DIVA $args
	    cd ..
	}
    }
}

#######################################################################
#### filterDocs
#
# Remove any doc directories that are not in DOCDIRS. Remove
# any package dirs that are not in the list of arguments.
#
proc filterDocs {DIVA args} {
    global debug silent

    if [file exists CVS] {
	error "Directory \"[pwd]\" is under CVS control -- cannot run"
    }
    set docdirs [getMakevar DOCDIRS]
    foreach dirpath [glob [file join $DIVA doc *]] {
	if ![file isdirectory $dirpath] {
	    continue
	}
	if { [lsearch -exact $docdirs [file tail $dirpath]] < 0 } {
	    puts "    Deleting \"[divaPath $DIVA $dirpath]\""
	    file delete -force $dirpath
	}
    }
    set pkgdirs {}
    foreach p $args {
	lappend pkgdirs [lindex [split $p .] 1]
    }
    foreach dirpath [glob [file join $DIVA doc packages *]] {
	if ![file isdirectory $dirpath] {
	    continue
	}
	if { [lsearch -exact $pkgdirs [file tail $dirpath]] < 0 } {
	    puts "    Deleting \"[divaPath $DIVA $dirpath]\""
	    file delete -force $dirpath
	}
    }
}







