# updateNavBars.tcl: update the nav-menus of top-level pages
#
# $Id: updateNavBars.tcl,v 1.9 2000/05/02 00:01:29 johnr Exp $
#
# Copyright (c) 1998-2000 The Regents of the University of California.
# All rights reserved. See the file COPYRIGHT for details.
#


### Assumed global variables:
# debug: print debug information if on
# silent: don't print anything if on

### Variables
#
set navbarFile [file join doc templates _navbar.html]
set footerFile [file join doc templates _footer.html]
set imagedir "images"
set ignoreDirs {
    api
    templates
    CVS
    SCCS
}

#######################################################################
#### updateNavBars
#
# Update the navigation bars of all pages except the top-level pages.
# If a file has a comment containing only the text "navbar," the contents
# of that element are replaced; otherwise, the navbar is inserted
# immediately following the body tag. The search for files starts
# in the current directory. All HTML files except those that already
# have a nav-menu, and those with names starting with an underscore,
# will be modified. The arguments can contain "-local," in which case
# subdirectories will not be entered. As a safety check, this proc
# will not proceed if the directory is not checked out from diva/doc.
#
# Added 10/20/98: Also add or substitute a footer using the _footer.html
# file, if the current file doesn't doesn't already contain a "custom
# footer."
#
proc updateNavBars {DIVA args} {
    set prefix ""
    if { $args != "" } {
	if { [lindex $args 0] == "-local" } {
	    set recursive 0
	    set args [lreplace $args 0 0]
	} else {
	    set recursive 1
	}
	if { $args != "" } {
	    set prefix [lindex $args 0]
	} else {
	    set prefix ""
	}
    }
    global debug silent
    global navbarFile footerFile imagedir
    
    # Print info
    set cwd [pwd]
    puts $cwd

    # Check that we are in a legal doc directory
    if ![file exists [file join CVS Repository]] {
	puts "$prefix$cwd -- not under CVS control"
	return
    }
    set fd [open [file join CVS Repository]]
    set path [string trim [read $fd]]
    if $debug {
	puts "${prefix}Checking repository path \"$path\""
    }
    set path [file split $path]
    close $fd
    set i [lsearch -exact $path diva]
    if { $i < 0 } {
	puts "$prefix$cwd -- not checked out from Diva"
	if $debug {
	    puts "$prefix  (unable to find \"diva\" in $path)"
	}
	exit 1
    }
    if { [lindex $path [expr $i+1]] != "doc" } {
	puts "$prefix$cwd -- not in Diva documentation tree"
	if $debug {
	    puts "$prefix  (expected to find \"doc\" at $i'th position in \{$path\})"
	}
	exit 1
    }

    # Find the distance from the root to here
    set p1 [file split $cwd]
    cd [file join $DIVA doc]
    if { [string first [pwd] $cwd] != 0 } {
	puts "The directory \"$cwd\" is not in the same tree as \"[pwd]\""
	exit 1
    }
    set p2 [file split [pwd]]
    cd $cwd
    if $debug {
	puts "Comparing\n    $p1\nwith\n    $p2"
    }
    set levels [expr [llength $p1] - [llength $p2]]
    set up {}
    for {set i 0} {$i < $levels} {incr i} {
	lappend up ".."
    }
    if { $up != "" } {
	set up [eval file join $up]
    }
    if $debug {
	puts "Path prefix is \"$up\""
    } 

    # Read the navbar template
    if $debug {
	puts "Reading navbar template from [file join $DIVA $navbarFile]"
    }
    set fd [open [file join $DIVA $navbarFile]]
    set navbar [read $fd]
    close $fd
    if $debug {
	puts "Raw template is\n$navbar"
    }

    # Modify the navbar text
    if { $up != "" } {
	regsub -all -nocase "src=\"" $navbar "src=\"$up/" navbar
	regsub -all -nocase "href=\"" $navbar "href=\"$up/" navbar
	if $debug {
	    puts "Modified template is\n$navbar"
	}
    }

    # Read the footer template
    if $debug {
	puts "Reading footer template from [file join $DIVA $footerFile]"
    }
    set fd [open [file join $DIVA $footerFile]]
    set footer [read $fd]
    close $fd
    if $debug {
	puts "Raw template is\n$footer"
    }

    # Modify the footer text
    if { $up != "" } {
	regsub -all -nocase "src=\"" $footer "src=\"$up/" footer
	regsub -all -nocase "href=\"" $footer "href=\"$up/" footer
	if $debug {
	    puts "Modified template is\n$footer"
	}
    }
 
    # Call helper
    _updateNavBars $prefix $navbar $footer
}


#######################################################################
#### _updateNavBars
#
# Helper method for updateNavBars{}.
#
proc _updateNavBars {prefix navbar footer} {
    global debug silent
    global imagedir ignoreDirs
    set cwd [pwd]

    # Check that we are in a legal doc directory
    if ![file exists [file join CVS Repository]] {
	puts "$prefix$cwd -- not under CVS control -- returning"
	return
    }

    # Do HTML files
    foreach file [glob -nocomplain *.html] {
	if !$silent {
	    puts -nonewline ${prefix}$file
	}
	if [string match {_*} $file] {
	    puts " -- ignoring"
	    continue
	}
	set fd [open $file]
	set text [read $fd]
	close $fd

	# Add the navbar only if there is no navmenu
	set rewrite 0
	if [hasComment text navmenu] {
	    puts -nonewline " -- has navmenu"
	} else {
	    if [hasComment text navbar] {
		# It has a navbar, so substitute it
		if $debug {
		    puts "\n${prefix}Substituting navbar"
		}
		set rewrite [substComment text navbar \n$navbar]
		
	    } elseif [hasTag text body] {
		# Immediately after the body tag
		if $debug {
		    puts "\n${prefix}Inserting navbar before <body>"
		}
		set s "<!--navbar-->\n$navbar<!--/navbar-->"
		insertAfter text body \n$s\n
		set rewrite 1
	    }
	}
	# Replace the footer if it has one, or insert one if it
	# doesn't have a custom footer
	if [hasComment text footer] {
	    # It has a footer, so substitute it
	    if $debug {
		puts "\n${prefix}Substituting footer"
	    }
	    set rewrite [substComment text footer \n$footer]
	    
	} elseif [hasComment text customfooter] {
	    # Do nothing
	    puts -nonewline " -- has customfooter"

	} elseif [hasTag text body] {
	    # Immediately before the body close tag
	    if $debug {
		puts "\n${prefix}Inserting footer before </body>"
	    }
	    set s "<!--footer-->\n$footer<!--/footer-->"
	    insertBefore text /body \n$s\n
	    set rewrite 1
	}
	
	# Rewrite the file
	if $rewrite {
	    set fd [open $file "w"]
	    puts $fd $text
	    close $fd
	    if !$silent {
		puts " -- ok"
	    }
	} else {
	    if !$silent {
		puts " -- navbar or body tag not found"
	    }
	}
    }

    # Modify the navbar text for next level
    regsub -all -nocase "src=\"" $navbar "src=\"../" navbar
    regsub -all -nocase "href=\"" $navbar "href=\"../" navbar
    if $debug {
	puts "${prefix}Template is now:\n$navbar"
    }
 
    # Modify the footer text for next level
    regsub -all -nocase "src=\"" $footer "src=\"../" footer
    regsub -all -nocase "href=\"" $footer "href=\"../" footer
    if $debug {
	puts "${prefix}Template is now:\n$footer"
    }
 
    # Do the subdirectories
    foreach file [glob -nocomplain *] {
	if { [file isdirectory $file] && \
		[lsearch -exact $ignoreDirs $file] < 0 } {
	    if !$silent {
		puts ${prefix}[file join $cwd $file]/
	    }
	    cd $file
	    _updateNavBars "$prefix    " $navbar $footer
	    cd $cwd
	}
    }
}







