/*
 * $Id: DragTutorial.java,v 1.13 2000/07/13 00:36:43 neuendor Exp $
 *
 * Copyright (c) 1998-2000 The Regents of the University of California.
 * All rights reserved. See the file COPYRIGHT for details.
 *
 */

package diva.canvas.tutorial;

import diva.canvas.CanvasPane;
import diva.canvas.Figure;
import diva.canvas.FigureLayer;
import diva.canvas.GraphicsPane;
import diva.canvas.JCanvas;

import diva.canvas.event.MouseFilter;

import diva.canvas.interactor.Interactor;
import diva.canvas.interactor.DragInteractor;
import diva.canvas.interactor.BoundedDragInteractor;

import diva.canvas.toolbox.BasicFigure;
import diva.canvas.toolbox.BasicEllipse;
import diva.canvas.toolbox.BasicRectangle;
import diva.canvas.toolbox.SwingWrapper;

import diva.gui.BasicFrame;

import java.awt.Color;
import java.awt.Shape;
import java.awt.geom.Rectangle2D;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import javax.swing.JButton;
import javax.swing.JLabel;

/** An example showing how to make figures draggable with interactors.
 * Each figure on the canvas can have its
 * interactor set or read with the methods setInteractor()
 * and getInteractor(). The interactor answers the
 * question "What does this figure do when I mouse on it?" In this
 * example, what the figure does is follow the mouse, and we are
 * setting up each figure's interactor with an object called
 * an <i>interactor</i> that makes it do this.
 *
 * <p>What an interactor does is simply respond to a series of
 * mouse press-drag-release events. Each method in this class
 * creates a different kind of interactor, which makes the figure
 * exhibit different behaviour when you mouse on it. See the method
 * comments for more explanation.
 *
 * <p>One point about interactors. In general, many figures 
 * will share a single interactor. For example, all nodes in a
 * graph editor might reference the "node role" object. This strategy
 * reduces the size of the figures, but more importantly, allows
 * the behaviour of a whole set of figure to be changed together
 * (by changing the behaviour of the interactor).
 *
 * @author John Reekie
 * @version $Revision: 1.13 $
 */
public class DragTutorial {

    // The JCanvas
    private JCanvas canvas;

    // The GraphicsPane
    private GraphicsPane graphicsPane;

    /** Create a JCanvas and put it into a window.
     */
    public DragTutorial () {
        canvas = new JCanvas();
        graphicsPane = (GraphicsPane)canvas.getCanvasPane();

        BasicFrame frame = new BasicFrame("Drag tutorial", canvas);
        frame.setSize(600,400);
        frame.setVisible(true);
    }

    /** Create a couple of simple figures and make them draggable.
     * Both figures are given the same interactor, which
     * means that they behave the same when you mouse on them.
     * The interactor used to move them is an instance of
     * DragInteractor.
     */
    public void createDraggableFigures () {
        FigureLayer layer = graphicsPane.getForegroundLayer();

        // Create the interactor to do the work.
        Interactor dragger = new DragInteractor();
        dragger.setMouseFilter(MouseFilter.defaultFilter);

        // Create a rectangle and make it draggable
        BasicFigure blue = new BasicRectangle(10.0,10.0,50.0,50.0,Color.blue);
        layer.add(blue);
        blue.setInteractor(dragger);

        // Create a circle and make it draggable
        BasicFigure red = new BasicEllipse(200.0,200.0,50.0,50.0,Color.red);
        layer.add(red);
        red.setInteractor(dragger);
    }

    /** Create another simple figures and make it draggable within
     * a region of the canvas. This example uses an instance of
     * BoundedDragInteractor to move the object.
     */
    public void createBoundedDraggableFigure () {
        FigureLayer layer = graphicsPane.getForegroundLayer();

        // Create the interactor and set it up for a 200x200 rectangle
        Rectangle2D bounds = new Rectangle2D.Double(100.0,100.0,200.0,200.0);
        Interactor boundedDragger = new BoundedDragInteractor(bounds);
        boundedDragger.setMouseFilter(MouseFilter.defaultFilter);

        // Create an outline rectangle that shows the boundaries
        graphicsPane.getOverlayLayer().add(bounds);

        // Create a green rectangle that stays inside the boundary
        BasicFigure green = new BasicFigure(
                new Rectangle2D.Double(110.0,110.0,50.0,50.0),
                Color.green);
        layer.add(green);
        green.setInteractor(boundedDragger);
    }

    /** Main function
     */
    public static void main (String argv[]) {
        DragTutorial ex = new DragTutorial();
        ex.createDraggableFigures();
        ex.createBoundedDraggableFigure();

        ex.canvas.repaint();
    }
}



